#ifndef lint
static char rcsid[] = "$Header: /mDNS/qconv/clipboard.c 15    01/01/25 11:22 Kato $";
#endif

/*
 * All of the documentation and software included in QCONV is copyrighted
 * by NTT Software Corporation.
 * 
 * Copyright 2000, 2001 NTT Software Corporation.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met: 
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE NTT SOFTWARE ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include <windowsx.h>
#include <stdio.h>

#include "qconv.h"

UCHAR *
StringCatAlloc(UCHAR *base, int *len, const UCHAR *str, BOOL newline)
{
	UCHAR	*new_ptr;

	int		len_base;
	int		len_str;
	int		len_nl;
	
	len_base = (len != NULL)? *len: ((base != NULL)? strlen(base): 0);
	len_str = (str != NULL)? strlen(str): 0;
	len_nl = (newline == TRUE)? 2: 0;				// 2 means length of CR+LF

	if ((new_ptr = realloc(base, len_base + len_str + len_nl + 1)) == NULL) {
		return base;
	}
	if (len_str > 0) {
		strcpy(&new_ptr[len_base], str);
	}
	if (newline == TRUE) {
		strcpy(&new_ptr[len_base + len_str], STR_NEWLINE);
	}
	if (len != NULL) {
		*len = len_base + len_str + len_nl;
	}

	return new_ptr;
}

int
InitializeConverter(void)
{
	static char	last_buff[MAX_MSG + 1] = { '\0' };
	char	msg[MAX_MSG + 1];
	char	str_msg[MAX_MSG + 1];

	LoadString(g_hinst, IDS_NOTICE_ENCODING_CHANGE, str_msg, sizeof(str_msg));
	sprintf(msg, str_msg, id_encoding_input, id_encoding);
	if (strcmp(last_buff, msg) == 0) {
		return 0;
	}
	AddErrorMessage(msg, TRUE);
	strcpy(last_buff, msg);
	if (!initialize_converter(id_encoding_input, id_encoding, NULL)) {
		return 1;
	}
	return 0;
}

int
InitializeNormalizer(HWND hDlg)
{
	char	*NormalizerNames[MAX_NORMALIZER];
	int		nNormalizers = 0;
	static char	last_buff[MAX_NORMALIZER_NAME + 1] = { 'd', 'm', 'y', '\0' };
	char	msg[MAX_NORMALIZER_NAME + 1];
	char	str_msg[MAX_MSG + 1];

	if (strcmp(last_buff, id_normalizer) == 0) {
		return 0;
	}
	strcpy(last_buff, id_normalizer);

	LoadString(g_hinst, IDS_NOTICE_NORMALIZER_CHANGED, str_msg, sizeof(str_msg));
	sprintf(msg, str_msg, id_normalizer);
	AddErrorMessage(msg, TRUE);
	nNormalizers = AnalyzeNormalizerNames(id_normalizer, NormalizerNames);
	if (!initialize_normalizer(NormalizerNames, nNormalizers)) {
		return 1;
	}
	return 0;
}

UINT
SetToClipBoard(HWND hDlg, UCHAR *str)
{
	HGLOBAL		copied_str;

	if ((copied_str = GlobalAlloc(GMEM_FIXED, strlen(str) + 1)) == NULL) {
		AddMainLastErrorMessage("GlobalAlloc: ");
		return 0;
	}
	strcpy((char *)copied_str, str);

	if (EmptyClipboard() == 0) {
		AddMainLastErrorMessage("EmptyClipboard: ");
		GlobalFree(copied_str);
		return 0;
	}

	if (SetClipboardData(CF_TEXT, copied_str) == 0) {
		AddMainLastErrorMessage("SetClipboardData: ");
		GlobalFree(copied_str);
		return 0;
	}

	return 1;
}

UCHAR *
ClipOneLine(const UCHAR *buffer, UCHAR **line_ptr, BOOL *newline)
{
	UCHAR	*next;
	int		len;

	next = strpbrk(buffer, STR_NEWLINE);
	if (next == NULL) {
		*line_ptr = strdup(buffer);
		*newline = FALSE;
		return NULL;				// End of buffer
	}
	*newline = TRUE;

	len = next - buffer;
	*line_ptr = malloc(len + 1);
	if (*line_ptr != NULL) {
		strncpy(*line_ptr, buffer, len);
		(*line_ptr)[len] = '\0';
	}

	if (strncmp(next, STR_NEWLINE, 2) == 0) {
		next++;
	}
	next++;

	if (*next == '\0') {
		return NULL;				// End of buffer
	}
	return next;					// Next Line
}

UCHAR *
ConvertOneLine(HWND hDlg, const UCHAR *string, const char *zld, int auto_zld)
{
	UCHAR		*new_buf = NULL;
	int			len_newbuf = 0;
	UCHAR		*msg_buf = NULL;
	int			len_msgbuf = 0;
	UCHAR		*string_ptr = (UCHAR *)string;
	UCHAR		*line_ptr;
	UCHAR		buffer[MAX_BUFFER + 1];
	BOOL		newline;
	char		in_msg[10];
	char		out_msg[10];

	LoadString(g_hinst, IDS_NOTICE_INPUT, in_msg, sizeof(in_msg));
	LoadString(g_hinst, IDS_NOTICE_OUTPUT, out_msg, sizeof(out_msg));

	while (string_ptr != NULL) {
		string_ptr = ClipOneLine(string_ptr, &line_ptr, &newline);
		if (line_ptr != NULL) {
			strncpy(buffer, line_ptr, MAX_BUFFER);
			buffer[MAX_BUFFER] = '\0';
			if (convert_string(buffer, MAX_BUFFER, zld, auto_zld) == 0) {	// no error
				if (id_result_only != TRUE) {
					msg_buf = StringCatAlloc(msg_buf, &len_msgbuf, in_msg, FALSE);
					msg_buf = StringCatAlloc(msg_buf, &len_msgbuf, line_ptr, TRUE);
				}
				new_buf = StringCatAlloc(new_buf, &len_newbuf, buffer, newline);
				if (id_result_only != TRUE) {
					msg_buf = StringCatAlloc(msg_buf, &len_msgbuf, out_msg, FALSE);
				}
				if (id_disp_hex_code) {
					char	*hex_dump;

					hex_dump = GetDumpList(buffer);
					msg_buf = StringCatAlloc(msg_buf, &len_msgbuf, hex_dump, TRUE);
					free(hex_dump);
				}
				else {
					msg_buf = StringCatAlloc(msg_buf, &len_msgbuf, buffer, TRUE);
				}
			}
			free(line_ptr);
		}
	}

	AddMainMessage(msg_buf, FALSE);
	free(msg_buf);

	return new_buf;
}

void
ConvertClipBoard(HWND hDlg, const char *zld, int auto_zld)
{
	HANDLE		copied_from = NULL;
	UCHAR		*local_string;
	UCHAR		*converted;

	if (OpenClipboard(hDlg) == 0) {
		AddMainLastErrorMessage("OpenClipboard: ");
		return;
	}

	if (GetClipboardOwner() == hDlg) {
		if (CloseClipboard() == 0) {
			AddMainLastErrorMessage("CloseClipboard: ");
		}
		return;
	}

	if (IsClipboardFormatAvailable(CF_UNICODETEXT)) {
		BOOL	Mapped;
		UINT	mblen;
		UINT	wclen;

		copied_from = GetClipboardData(CF_UNICODETEXT);
		wclen = wcslen((const wchar_t *)copied_from);
		mblen = WideCharToMultiByte(CP_OEMCP,
									0,
									(LPCWSTR)copied_from,
									wclen,
									NULL,
									0,
									NULL,
									&Mapped);
		if (mblen == 0) {
			AddMainLastErrorMessage("WideCharToMultiByte: ");
			goto error;
		}
		local_string = malloc(mblen + 1);
		if (local_string == NULL) {
			AddMainLastErrorMessage("malloc: ");
			goto error;
		}
		mblen = WideCharToMultiByte(CP_OEMCP,
									0,
									(LPCWSTR)copied_from,
									wclen,
									local_string,
									mblen + 1,
									NULL,
									&Mapped);
		if (mblen == 0) {
			AddMainLastErrorMessage("WideCharToMultiByte: ");
			goto error;
		}
		local_string[mblen] = '\0';
	}
	else if (IsClipboardFormatAvailable(CF_TEXT)) {
		copied_from = GetClipboardData(CF_TEXT);
		if ((local_string = malloc(strlen((const char *)copied_from) + 1)) == NULL) {
			AddMainLastErrorMessage("malloc: ");
			goto error;
		}
		strcpy(local_string, (const char *)copied_from);
	}
	if (copied_from == NULL) {
		char	message[MAX_MSG + 1];

		LoadString(g_hinst, IDS_EMPTY_CLIPBOARD, message, sizeof(message));
		AddErrorMessage(message, TRUE);
		goto error;
	}

	converted = ConvertOneLine(hDlg, (UCHAR *)local_string, zld, auto_zld);

	if ((GetClipboardOwner() != hMainDlg)  && id_copy_to_clipboard && (converted != NULL)) {
		SetToClipBoard(hDlg, converted);
	}

error:
	if (CloseClipboard() == 0) {
		AddMainLastErrorMessage("CloseClipboard: ");
	}
	if (converted != NULL) {
		free(converted);
		converted = NULL;
	}
	if (local_string != NULL) {
		free(local_string);
		local_string = NULL;
	}
}
