#ifndef lint
static char rcsid[] = "$Header: /mDNS/qconv/convert.c 11    01/01/25 11:22 Kato $";
#endif

/*
 * All of the documentation and software included in QCONV is copyrighted
 * by NTT Software Corporation.
 * 
 * Copyright 2000, 2001 NTT Software Corporation.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met: 
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE NTT SOFTWARE ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include <windowsx.h>
#include <stdio.h>
#include <ctype.h>

#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <mdn/result.h>
#include <mdn/converter.h>
#include <mdn/normalizer.h>
#include <mdn/localencoding.h>
#include <mdn/utf8.h>
#include <mdn/selectiveencode.h>

#include "qconv.h"
#include "util.h"

#ifdef MDNKIT_1_1_OR_OLDER
#ifndef MDN_RACE_ENCODING_NAME
#define MDN_RACE_ENCODING_NAME "RACE"
#endif

#ifndef MDN_LACE_ENCODING_NAME
#define MDN_LACE_ENCODING_NAME "LACE"
#endif

#ifndef MDN_RACE_PREFIX
#define MDN_RACE_PREFIX "bq--"
#endif

#ifndef MDN_LACE_PREFIX
#define MDN_LACE_PREFIX "bq--"
#endif
#endif	// MDNKIT_1_1_OR_OLDER

mdn_converter_t	conv_in_ctx;	/* input converter */
mdn_converter_t	conv_out_ctx;	/* output converter */
mdn_normalizer_t	norm_ctx;	/* normalizer */

int				line_number = 0;

mdn_result_t	encode_region(const char *region, char *to, int tolen,
							  const char *zld, int auto_zld);
mdn_result_t	selective_encode(char *from, char *to, int tolen,
								 const char *zld, int auto_zld);

int				errormsg(const char *format, ...);

int
convert_string(char *in_str, unsigned int sizeof_in_str, const char *zld, int auto_zld)
{
	mdn_result_t	r;
	char			tmp_buf[MAX_BUFFER + 1];

#ifdef MDNKIT_1_1_OR_OLDER
	if (strcmp(mdn_converter_localencoding(conv_in_ctx), MDN_RACE_ENCODING_NAME) == 0) {
		/*
		 * Find portions encoded in RACE using RACE's prefix
		 * "bq--", and selectively decode those portions.
		 */
		r = selective_decode(MDN_RACE_PREFIX, in_str, tmp_buf, MAX_BUFFER);
	} else if (strcmp(mdn_converter_localencoding(conv_in_ctx), MDN_LACE_ENCODING_NAME) == 0) {
		/*
		 * Find portions encoded in LACE using LACE's prefix
		 * "bq--", and selectively decode those portions.
		 */
		r = selective_decode(MDN_LACE_PREFIX, in_str, tmp_buf, MAX_BUFFER);
	} else {
		/*
		 * Convert input line to UTF-8.
		 */
		r = mdn_converter_localtoutf8(conv_in_ctx, in_str, tmp_buf, MAX_BUFFER);
	}
#else
	if (strcmp(mdn_converter_localencoding(conv_in_ctx), id_encoding_local) != 0) {
		/*
		 * Find portions in ACE, and selectively decode it.
		 */
		r = selective_decode(in_str, tmp_buf, MAX_BUFFER);
	} else {
		/*
		 * Convert input line to UTF-8.
		 */
		r = mdn_converter_localtoutf8(conv_in_ctx, in_str, tmp_buf, MAX_BUFFER);
	}
#endif
	if (r != mdn_success) {
		char	msg[MAX_MSG + 1];

		LoadString(g_hinst, IDS_ERR_CONVERSION, msg, sizeof(msg));
		errormsg(msg, mdn_result_tostring(r));
		return (1);
	}
	if (!mdn_utf8_isvalidstring(tmp_buf)) {
		char	msg[MAX_MSG + 1];

		LoadString(g_hinst, IDS_ERR_CONVERSION, msg, sizeof(msg));
		errormsg(msg, "not valid string");
		return (1);
	}

	/*
	 * Normalize and convert to the output codeset.
	 */
//	if (selective) {
		r = selective_encode(tmp_buf, in_str, sizeof_in_str, zld, auto_zld);
//	}
//	else {
//		r = encode_region(tmp_buf, in_str, sizeof_in_str, zld, auto_zld);
//	}
	if (r != mdn_success) {
		return (1);
	}

	return (0);
}

int
errormsg(const char *format, ...)
{
	va_list arglist;
	char	buff[MAX_BUFFER + 1];
	int		ret;

	va_start(arglist, format);
	ret = _vsnprintf(buff, (size_t)MAX_BUFFER, format, arglist);
	buff[ret] = '\0';
	if ((ret > 1) && (buff[ret - 1] == '\n')) {
		buff[ret - 1] = '\0';
	}
	va_end(arglist);

	AddErrorMessage(buff, TRUE);

	return ret;
}
