#ifndef lint
static char rcsid[] = "$Header: /mDNS/qconv/maindlg.c 12    01/01/25 11:22 Kato $";
#endif

/*
 * All of the documentation and software included in QCONV is copyrighted
 * by NTT Software Corporation.
 * 
 * Copyright 2000, 2001 NTT Software Corporation.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met: 
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE NTT SOFTWARE ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include <windowsx.h>
#include <stdio.h>

#include "qconv.h"

HWND	hMainDlg;
char	*application_name = "qconv";

UCHAR	id_encoding_input[MAX_NAME + 1];		// IDC_INPUT_ENCODING
UCHAR	id_encoding[MAX_NAME + 1];				// IDC_OUTPUT_ENCODING
UCHAR	id_encoding_other[MAX_NAME + 1];		// IDC_CONV_OTHER
UCHAR	id_encoding_local[MAX_NAME + 1];		// Read Only
BOOL	id_show_main;							// IDC_SHOW_MAIN
UCHAR	id_zld[MAX_NAME + 1];					// IDC_ZLD_STRING
BOOL	id_auto_zld;							// IDC_SET_AUTO_ZLD
UCHAR	id_normalizer[MAX_NORMALIZER_NAME + 1];	// IDD_NORMALIZE
UCHAR	id_normalizer_order[MAX_NORMALIZER_NAME + 1];// Read Only
BOOL	id_copy_to_clipboard;					// IDC_COPY_TO_CB
BOOL	id_always_on_top;						// IDC_ALWAYS_ON_TOP
int		id_edit_buffer_len;						// IDC_MAX_MESSAGE_LEN
BOOL	id_result_only;							// IDC_RESULT_ONLY | IDC_RESULT_AND_INPUT
BOOL	id_disp_hex_code;						// IDC_HEX_CODE
BOOL	id_on_left_click;						// IDC_ON_LEFT_CLICK | IDC_CONTINUOUS
RECT	id_pos;									// Main Window Position
BOOL	id_disable_error;						// IDC_DISABLE_ERROR

UCHAR	*message_buffer = NULL;
int		message_len = 0;

char	module_name[_MAX_PATH];
char	inifile_name[_MAX_PATH];

void
AddErrorMessage(UCHAR *str, BOOL newline)
{
	if (id_disable_error == FALSE) {
		AddMainMessage(str, newline);
	}
}

void
AddMainMessage(UCHAR *str, BOOL newline)
{
	UCHAR	*ptr;
	UCHAR	*tmp_ptr;

	message_buffer = StringCatAlloc(message_buffer, &message_len, str, newline);

	if ((id_edit_buffer_len != 0)
		&& (message_len > (id_edit_buffer_len * 1024))) {
		ptr = message_buffer + (message_len - (id_edit_buffer_len * 1024));
		tmp_ptr = strpbrk(ptr, STR_NEWLINE);
		if ((tmp_ptr != NULL) && (strncmp(tmp_ptr, STR_NEWLINE, 2) == 0)) {
			ptr = tmp_ptr + 2;
		}
		memmove(message_buffer, ptr, strlen(ptr) + 1);
		message_buffer = realloc(message_buffer, strlen(message_buffer) + 1);
	}

	SetDlgItemText(hMainDlg, IDC_MAIN_MSG, (LPSTR)message_buffer);
	SendMessage(GetDlgItem(hMainDlg, IDC_MAIN_MSG), EM_LINESCROLL, 0, 100000);
}

void
EraseMainMessage()
{
	free(message_buffer);
	message_buffer = NULL;
	message_len = 0;

	SetDlgItemText(hMainDlg, IDC_MAIN_MSG, (LPSTR)message_buffer);
	SendDlgItemMessage(hMainDlg, IDC_MAIN_MSG, EM_LINESCROLL, 0, 100000);
}

void
GetMainMessage()
{
	char	*ptr;
	int		len;

	len = GetWindowTextLength(GetDlgItem(hMainDlg, IDC_MAIN_MSG));

	if ((ptr = malloc(len + 1)) != NULL) {
		free(message_buffer);
		message_buffer = ptr;
		message_len = len;

		GetDlgItemText(hMainDlg, IDC_MAIN_MSG, (LPSTR)message_buffer, message_len + 1);
		SendDlgItemMessage(hMainDlg, IDC_MAIN_MSG, EM_LINESCROLL, 0, 100000);
	}
}

void
ResizeMainMessage(HWND hDlg)
{
	RECT	rc;
	POINT	pos;
	HWND	hMainMsg;

	hMainMsg = GetDlgItem(hDlg, IDC_MAIN_MSG);
	GetWindowRect(hMainMsg, &rc);
	pos.x = rc.left;
	pos.y = rc.top;
	ScreenToClient(hDlg, &pos);

	GetClientRect(hDlg, &rc);
	MoveWindow(hMainMsg, rc.left, rc.top + pos.y,
			   rc.right - rc.left, rc.bottom - rc.top - pos.y, TRUE);
}

static void
get_module_name(void)
{
	char	drive[_MAX_DRIVE];
	char	dirname[_MAX_PATH];

    GetModuleFileName(NULL, module_name, _MAX_PATH);
    _splitpath(module_name, drive, dirname, NULL, NULL);
	_makepath(inifile_name, drive, dirname, application_name, "ini");
}

UINT
QGetProfileString(char *name, char *def_value, char *buffer, int len)
{
	return GetPrivateProfileString(application_name,
								   name,
								   def_value,
								   buffer,
								   len,
								   inifile_name);
}

UINT
QGetProfileInt(char *name, int def_value)
{
	return GetPrivateProfileInt(application_name,
								name,
								def_value,
								inifile_name);
}

BOOL
QGetProfileBool(char *name, BOOL def_value)
{
	UINT	iRet;
	char	buffer[MAX_MSG + 1];

	iRet = GetPrivateProfileString(application_name,
								   name,
								   def_value? "TRUE": "FALSE",
								   buffer,
								   MAX_MSG,
								   inifile_name);
	return IsTrueString(buffer)? TRUE: FALSE;
}

void
ReadInitFile(HWND hDlg)
{
	UINT	iRet;
	char	local_encode[MAX_NAME + 1];

	get_module_name();

	iRet = QGetProfileString(PRF_OUTPUT_ENCODING,
							 DEFAULT_ENCODING_NAME, id_encoding,
							 MAX_NAME);
	iRet = QGetProfileString(PRF_OTHER_ENCODING, "",
							 id_encoding_other, MAX_NAME);

	LoadString(g_hinst, IDS_DEFAULT_LOCAL_ENCODING,
			   local_encode, sizeof(local_encode));
	iRet = QGetProfileString(PRF_LOCAL_ENCODING, local_encode,
							 id_encoding_local, MAX_NAME);
	iRet = QGetProfileString(PRF_INPUT_ENCODING,
							 id_encoding_local, id_encoding_input,
							 MAX_NAME);
	iRet = QGetProfileString(PRF_ZLD, "", id_zld, MAX_NAME);

	id_show_main = QGetProfileBool(PRF_SHOW_MAIN, TRUE);
	id_auto_zld = QGetProfileBool(PRF_AUTO_ZLD, FALSE);

	iRet = QGetProfileString(PRF_NORMALIZER, "", id_normalizer, MAX_NORMALIZER_NAME);
	CheckNormalizerOrder(id_normalizer, FALSE);

	id_copy_to_clipboard = QGetProfileBool(PRF_COPY_TO_CLIPBOARD, FALSE);
	id_always_on_top = QGetProfileBool(PRF_ALWAYS_ON_TOP, FALSE);

	id_edit_buffer_len = QGetProfileInt(PRF_MAX_EDIT_BUFFER_LEN, DEFAULT_EDIT_BUFFER_LEN);

	id_result_only = QGetProfileBool(PRF_RESULT_ONLY, FALSE);
	id_disp_hex_code = QGetProfileBool(PRF_HEX_CODE, FALSE);
	id_on_left_click = QGetProfileBool(PRF_ON_LEFT_CLICK, TRUE);

	id_pos.left = QGetProfileInt(PRF_WINDOW_POS_X, 0);
	id_pos.top = QGetProfileInt(PRF_WINDOW_POS_Y, 0);

	id_disable_error = QGetProfileBool(PRF_DISABLE_ERROR, FALSE);

	iRet = QGetProfileString(PRF_NORMALIZER_ORDER, "", id_normalizer_order, MAX_NORMALIZER_NAME);
	CheckNormalizerOrder(id_normalizer_order, TRUE);
}

void
WritebackInitFile(HWND hDlg)
{
	char	buffer[MAX_MSG + 1];

	WritePrivateProfileString(application_name, PRF_INPUT_ENCODING, id_encoding_input, inifile_name);
	WritePrivateProfileString(application_name, PRF_OUTPUT_ENCODING, id_encoding, inifile_name);
	WritePrivateProfileString(application_name, PRF_OTHER_ENCODING, id_encoding_other, inifile_name);
	WritePrivateProfileString(application_name, PRF_LOCAL_ENCODING, id_encoding_local, inifile_name);
	WritePrivateProfileString(application_name, PRF_SHOW_MAIN, id_show_main? "True": "False", inifile_name);
	WritePrivateProfileString(application_name, PRF_ZLD, id_zld, inifile_name);
	WritePrivateProfileString(application_name, PRF_AUTO_ZLD, id_auto_zld? "True": "False", inifile_name);
	WritePrivateProfileString(application_name, PRF_NORMALIZER, id_normalizer, inifile_name);
	WritePrivateProfileString(application_name, PRF_COPY_TO_CLIPBOARD,
							  id_copy_to_clipboard? "True": "False", inifile_name);
	WritePrivateProfileString(application_name, PRF_ALWAYS_ON_TOP,
							  id_always_on_top? "True": "False", inifile_name);

	sprintf(buffer, "%d", id_edit_buffer_len);
	WritePrivateProfileString(application_name, PRF_MAX_EDIT_BUFFER_LEN, buffer, inifile_name);

	WritePrivateProfileString(application_name, PRF_RESULT_ONLY,
							  id_result_only? "True": "False", inifile_name);
	WritePrivateProfileString(application_name, PRF_HEX_CODE,
							  id_disp_hex_code? "True": "False", inifile_name);
	WritePrivateProfileString(application_name, PRF_ON_LEFT_CLICK,
							  id_on_left_click? "True": "False", inifile_name);

	sprintf(buffer, "%d", id_pos.left);
	WritePrivateProfileString(application_name, PRF_WINDOW_POS_X, buffer, inifile_name);
	sprintf(buffer, "%d", id_pos.top);
	WritePrivateProfileString(application_name, PRF_WINDOW_POS_Y, buffer, inifile_name);

	WritePrivateProfileString(application_name, PRF_DISABLE_ERROR,
							  id_disable_error? "True": "False", inifile_name);

	WritePrivateProfileString(application_name, PRF_NORMALIZER_ORDER,
							  id_normalizer_order, inifile_name);
}

void
QCMoveWindow()
{
	WINDOWPLACEMENT	wplace;

	wplace.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement(hMainDlg, &wplace);
	MoveWindow(hMainDlg, id_pos.left, id_pos.top,
			   wplace.rcNormalPosition.right - wplace.rcNormalPosition.left,
			   wplace.rcNormalPosition.bottom - wplace.rcNormalPosition.top, TRUE);
}

void
QCSetWindowPos(void)
{
	if (id_always_on_top) {
		SetWindowPos(hMainDlg, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	}
	else {
		SetWindowPos(hMainDlg, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	}
}

HWND	hNextCBChain;

void
QCSetClipboardViewer(void)
{
	hNextCBChain = SetClipboardViewer(hMainDlg);
}

BOOL
IsWindowsNT(void)
{
	BOOL				bRet;
	OSVERSIONINFO		OSVersion;

	OSVersion.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	bRet = GetVersionEx(&OSVersion);
	if (bRet == 0) {
		return FALSE;
	}

	if (OSVersion.dwPlatformId == VER_PLATFORM_WIN32_NT) {
		return TRUE;
	}
	return FALSE;
}

void
Initialize(HWND hDlg)
{
	hMainDlg = hDlg;

	ReadInitFile(hDlg);

	InitializeConverter();
	InitializeNormalizer(hDlg);

	TrayMessage(hDlg, NIM_ADD, NULL, NULL);
	SetTrayIcon(hDlg);
	SetTimer(hDlg, 1, 1000, NULL);

	QCMoveWindow();
	QCSetWindowPos();
	QCSetClipboardViewer();

	ShowWindow(hDlg, id_show_main? SW_SHOW: SW_HIDE);
}

BOOL CALLBACK
QConvDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static BOOL		initialized = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		Initialize(hDlg);
		initialized = TRUE;
		break;

	case WM_TIMER:
		if (wParam == 1) {
			ShowWindow(hDlg, id_show_main? SW_SHOW: SW_HIDE);
		}
		break;

	case WM_SHOWWINDOW:
		if ((BOOL)wParam != id_show_main) {
			ShowWindow(hDlg, id_show_main? SW_SHOW: SW_HIDE);
		}
		break;

	case WM_DRAWITEM:
		return IconDrawItem((LPDRAWITEMSTRUCT)lParam);
		break;

	case WM_DESTROY:
		WritebackInitFile(hDlg);
		TrayMessage(hDlg, NIM_DELETE, NULL, NULL);
		break;

	case WM_SIZE:
		ResizeMainMessage(hDlg);
	case WM_MOVE:
//	case WM_WINDOWPOSCHANGED:
		if (initialized == TRUE) {
			GetWindowRect(hDlg, &id_pos);
		}
		break;

	case WM_RBUTTONDOWN:
		QCPopupMenu(hDlg, lParam);
		break;

	case WM_CHANGECBCHAIN:
		hNextCBChain = (HWND)lParam;
		if (hNextCBChain != NULL) {
			SendMessage(hNextCBChain, WM_CHANGECBCHAIN, wParam, lParam);
		}
		break;

	case WM_DRAWCLIPBOARD:
		if (hNextCBChain != NULL) {
			SendMessage(hNextCBChain, WM_DRAWCLIPBOARD, (DWORD)0, (long)0);
		}
		if (id_on_left_click != TRUE) {
			ConvertClipBoard(hDlg, id_zld, id_auto_zld);
		}
		break;

	case WM_COMMAND:
		switch (GET_WM_COMMAND_ID(wParam, lParam))
		{
		case IDCANCEL:
			ShowWindow(hDlg, SW_HIDE);
			id_show_main = FALSE;
			break;

		case IDC_CLOSE:
			ShowWindow(hDlg, SW_HIDE);
			id_show_main = FALSE;
			break;

		case IDC_QUIT:
			EndDialog(hDlg, TRUE);
			break;

		case IDC_SHOW_MAIN:
			id_show_main = !id_show_main;
			ShowWindow(hDlg, id_show_main? SW_SHOW: SW_HIDE);
			break;

		case IDC_CONV_UTF8:
		case IDC_CONV_UTF5:
		case IDC_CONV_RACE:
		case IDC_CONV_OTHER:
			SetEncodingFromMenu(hDlg, GET_WM_COMMAND_ID(wParam, lParam));
			break;

		case IDC_ABOUT:
			DialogBox(g_hinst, MAKEINTRESOURCE(IDD_ABOUT), hDlg, AboutDlgProc);
			break;

		case IDC_SETTING:
			DialogBox(g_hinst, MAKEINTRESOURCE(IDD_DLG_SETTING), hDlg, SettingDlgProc);
			break;

		case IDC_NORMALIZE:
			DialogBox(g_hinst, MAKEINTRESOURCE(IDD_NORMALIZE), hDlg, NormalizerDlgProc);
			break;

		case IDC_COPY_TO_CB:
			id_copy_to_clipboard = !id_copy_to_clipboard;
			break;

		case IDC_ON_LEFT_CLICK:
			id_on_left_click = TRUE;
			break;

		case IDC_CONTINUOUS:
			id_on_left_click = FALSE;
			break;

		case IDC_ON_TOP:
			id_always_on_top = !id_always_on_top;
			if (id_always_on_top) {
				SetWindowPos(hDlg, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
			}
			else {
				SetWindowPos(hDlg, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
			}
			break;

		case IDC_CLEAR:
			EraseMainMessage(hDlg);
			break;

		case IDC_CONVERT:
			ConvertClipBoard(hDlg, id_zld, id_auto_zld);
			break;

		case IDC_MAIN_MSG:
			switch (HIWORD(wParam)) {
			case EN_CHANGE:
				GetMainMessage();
				break;
			}
			break;
		}
		break;

	case MYWM_NOTIFYICON:
		switch (lParam)
		{
		case WM_LBUTTONDOWN:
			if (id_on_left_click) {
				ConvertClipBoard(hDlg, id_zld, id_auto_zld);
			}
			break;

		case WM_RBUTTONDOWN:
			QCPopupMenu(hDlg, lParam);
			break;

		default:
			break;
		}
		break;

	default:
		return(FALSE);
	}

	return(TRUE);
}
