#ifndef lint
static char rcsid[] = "$Header: /mDNS/qconv/misc.c 11    01/01/25 11:22 Kato $";
#endif

/*
 * All of the documentation and software included in QCONV is copyrighted
 * by NTT Software Corporation.
 * 
 * Copyright 2000, 2001 NTT Software Corporation.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met: 
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE NTT SOFTWARE ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include <windowsx.h>
#include <stdio.h>

#include "qconv.h"

ETABLE	encoding_table[] = {
	{ IDC_CONV_UTF8, "UTF-8", FALSE },
	{ IDC_CONV_UTF5, "UTF-5", FALSE },
	{ IDC_CONV_RACE, "RACE", TRUE },
	{ IDC_CONV_LACE, "LACE", TRUE },
	{ IDC_CONV_BRACE, "BRACE", TRUE }
};

#define	SIZEOF_NORMALIZER_ETABLE	(sizeof(normalizer_table)/sizeof(ETABLE))

ETABLE	normalizer_table[] = {
	{ IDC_NORM_JA_ZENKAKU, "ja-kana-fullwidth", FALSE },
	{ IDC_NORM_ALNUM_HALFWIDTH, "ja-alnum-halfwidth", FALSE },
	{ IDC_NORM_COMPOSE_VOICED_SOUND, "ja-compose-voiced-sound", FALSE },
	{ IDC_NORM_MINUS_HACK, "ja-minus-hack", FALSE },
	{ IDC_NORM_DELIMITER_HACK, "ja-delimiter-hack", FALSE },
	{ IDC_NORM_UNICODE_UPPERCASE, "unicode-uppercase", FALSE },
	{ IDC_NORM_UNICODE_LOWERCASE, "unicode-lowercase", FALSE },
	{ IDC_NORM_NORMALIZATION_C, "unicode-form-c", FALSE },
	{ IDC_NORM_NORMALIZATION_KC, "unicode-form-kc", FALSE },
	{ IDC_NORM_ASCII_UPPERCASE, "ascii-uppercase", FALSE },
	{ IDC_NORM_ASCII_LOWERCASE, "ascii-lowercase", FALSE }
};

char *strings[] = {
	"true", "on", "1", "ok", "yes", NULL
};

BOOL
IsTrueString(char *str)
{
	char	**ptr;

	for (ptr = strings; *ptr != NULL; ptr++) {
		if (stricmp(*ptr, str) == 0) {
			return TRUE;
		}
	}
	return FALSE;
}

UINT
GetIDForString(ETABLE *table, int size, UCHAR *name)
{
	int		i;

	for (i = 0; i < size; i++) {
		if (stricmp(table[i].name, name) == 0) {
			return table[i].menu_item;
		}
	}
	return IDC_NOMENUITEM;
}

UCHAR *
GetStringForID(ETABLE *table, int size, UINT menu_item)
{
	int		i;

	for (i = 0; i < size; i++) {
		if (table[i].menu_item == menu_item) {
			return table[i].name;
		}
	}
	return NULL;
}

void
SetEncodingFromMenu(HWND hDlg, UINT return_id)
{
	UCHAR	*new_encoding_name;

	new_encoding_name = GetNameForMenuItem(return_id);
	if (new_encoding_name != NULL) {
		strcpy(id_encoding, new_encoding_name);
	}
	else {
		// IDC_CONV_OTHER
		strcpy(id_encoding, id_encoding_other);
	}
	InitializeConverter();
	InitializeNormalizer(hDlg);
}

#define	GetMenuItemForNormalizer(name)	\
			GetIDForString(normalizer_table, SIZEOF_NORMALIZER_ETABLE, name)
#define	GetNormalizerForMenuItem(id)	\
			GetStringForID(normalizer_table, SIZEOF_NORMALIZER_ETABLE, id)

void
GetNormalizerNames(HWND hNormalizerDlg, char *normalizer)
{
	int		count = 0;
	int		i;

	normalizer[0] = '\0';

	for (i = 0; i < SIZEOF_NORMALIZER_ETABLE; i++) {
		if (IsDlgButtonChecked(hNormalizerDlg, normalizer_table[i].menu_item) == BST_CHECKED) {
			if (count > 0) {
				strcat(normalizer, STR_NORMALIZER_DELIMTER);
			}
			strcat(normalizer, normalizer_table[i].name);
		}
		count++;
	}

	CheckNormalizerOrder(normalizer, FALSE);
}

void
SetNormalizerNames(HWND hNormalizerDlg, const char *normalizer)
{
	char	*NormalizerNames[MAX_NORMALIZER];
	int		nNormalizers = 0;
	int		i;

	for (i = 0; i < SIZEOF_NORMALIZER_ETABLE; i++) {
		CheckDlgButton(hNormalizerDlg, normalizer_table[i].menu_item, BST_UNCHECKED);
	}
	nNormalizers = AnalyzeNormalizerNames(normalizer, NormalizerNames);
	for (i = 0; i < nNormalizers; i++) {
		CheckDlgButton(hNormalizerDlg,
					   GetMenuItemForNormalizer(NormalizerNames[i]),
					   BST_CHECKED);
	}
}

int
AnalyzeNormalizerNames(const char *normalizer, char **NormalizerNames)
{
	int		i;
	int		index = 0;
	char	*st_ptr, *en_ptr;
	char	entry[MAX_NORMALIZER_NAME];

	st_ptr = (char *)normalizer;

	while (*st_ptr != '\0') {
		en_ptr = strchr(st_ptr, ',');
		if (en_ptr != NULL) {
			strncpy(entry, st_ptr, en_ptr - st_ptr);
			entry[en_ptr - st_ptr] = '\0';
			st_ptr = en_ptr + 1;
		}
		else {
			strcpy(entry, st_ptr);
			st_ptr += strlen(st_ptr);
		}
		for (i = 0; i < SIZEOF_NORMALIZER_ETABLE; i++) {
			if (strcmp(entry, normalizer_table[i].name) == 0) {
				break;
			}
		}
		if (i < SIZEOF_NORMALIZER_ETABLE) {
			NormalizerNames[index++] = normalizer_table[i].name;
		}
	}

	for (i = index; i < SIZEOF_NORMALIZER_ETABLE; i++) {
		NormalizerNames[i] = NULL;
	}
	return index;
}

char *
GetDefaultNormalizerOrder(void)
{
	static char		ret[MAX_NORMALIZER_NAME + 1];
	int				i;

	strcpy(ret, normalizer_table[0].name);

	for (i = 1; i < SIZEOF_NORMALIZER_ETABLE; i++) {
		strcat(ret, STR_NORMALIZER_DELIMTER);
		strcat(ret, normalizer_table[i].name);
	}

	return ret;
}

void
CheckNormalizerOrder(char *normalizer, BOOL completion)
{
	char	*nz[MAX_NORMALIZER];
	int		num_nz;
	int		count = 0;
	int		i, j;

	num_nz = AnalyzeNormalizerNames(normalizer, nz);
	for (i = 0; i < SIZEOF_NORMALIZER_ETABLE; i++) {
		normalizer_table[i].flag = FALSE;
	}

	normalizer[0] = '\0';

	for (i = 0; i < num_nz; i++) {
		for (j = 0; j < SIZEOF_NORMALIZER_ETABLE; j++) {
			if (strcmp(nz[i], normalizer_table[j].name) == 0) {
				if (count > 0) {
					strcat(normalizer, STR_NORMALIZER_DELIMTER);
				}
				strcat(normalizer, normalizer_table[j].name);
				normalizer_table[j].flag = TRUE;
				count++;
				break;
			}
		}
	}
	if (completion) {
		for (j = 0; j < SIZEOF_NORMALIZER_ETABLE; j++) {
			if (normalizer_table[j].flag == FALSE) {
				if (count > 0) {
					strcat(normalizer, STR_NORMALIZER_DELIMTER);
				}
				strcat(normalizer, normalizer_table[j].name);
				count++;
				if (count >= SIZEOF_NORMALIZER_ETABLE) {
					break;
				}
			}
		}
	}
}

void
AddMainLastErrorMessage(char *str)
{
	LPVOID lpMsgBuf;

	AddErrorMessage(str, FALSE);
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
				  NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
				  (LPTSTR) &lpMsgBuf, 0, NULL);
	AddErrorMessage((char *)lpMsgBuf, FALSE);
	LocalFree(lpMsgBuf);
}

// Dump line format
// 01234567891123456789212345678931234567894123456789512345678961234567897123456789
// oooo: xx xx xx xx xx xx xx xx  xx xx xx xx xx xx xx xx : cccccccc cccccccc
// o: offset	x: dump data	c: charactor (if printable)

#define	MAX_LINE	(4 + 2 + (3 * 16) + 1 + 2 + 16 + 1 + 2)		// line length (+CR/LF)

static char		hex[] = "0123456789abcdef";

void
itox(char *ptr, unsigned int data, int len)
{
	int				i;
	unsigned int	x;

	for (i = len - 1; i >= 0; i--) {
		x = data % 16;
		data /= 16;
		ptr[i] = hex[x];
	}
}

#define off_hex(j)		(((j) * 3) + (((j) >= 8)? 1: 0) + 6)
#define off_chr(j)		((j) + (((j) >= 8)? 1: 0) + 57)

char *
GetDumpList(char *str)
{
	int		len;
	int		lines;
	char	*ptr;
	int		i, j;
	char	*ln;

	len = strlen(str);
	lines = (len + 15) / 16;
	if ((ptr = malloc(MAX_LINE * lines + 3)) == NULL) {
		return NULL;
	}
	*ptr = '\0';

	for (i = 0; i < lines; i++) {
		strcpy(&ptr[MAX_LINE * i], STR_NEWLINE);
		ln = ptr + MAX_LINE * i + 2;
		memset(ln, ' ', MAX_LINE - 2);
		ln[MAX_LINE - 2] = '\0';

		itox(ln, i * 16, 4);
		ln[4] = ':';
		ln[55] = ':';

		for (j = 0; (j < 16) && ((i * 16 + j) < len); j++) {
			itox(ln + off_hex(j), (unsigned int)(str[i * 16 + j] & 0xff), 2);
			ln[off_chr(j)] = isprint(str[i * 16 + j])? str[i * 16 + j]: '.';
		}
	}

	return ptr;
}
